/*============================================================================
  File:     TempdbDebug.sql

  Summary:  Simulate using tempdb space

  Date:     March 2011

  SQL Server Versions:
		10.0.2531.00 (SS2008 SP1)
		9.00.4035.00 (SS2005 SP3)
------------------------------------------------------------------------------
  Written by Paul S. Randal, SQLskills.com

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you give due credit.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- Start tempdb workload running

SELECT * FROM sys.dm_db_file_space_usage;
GO

SELECT * FROM sys.dm_db_task_space_usage;
GO


SELECT 
	GETDATE () AS [Date],
	tsu.session_id AS [SessionID],
	tsu.exec_context_id AS [ExecContextID],
	(tsu.user_objects_alloc_page_count - tsu.user_objects_dealloc_page_count) AS [UserPages],
	ROUND (CONVERT (FLOAT, (tsu.user_objects_alloc_page_count - tsu.user_objects_dealloc_page_count) * 8) / 1024.0, 2) AS [UserMB],
	(tsu.internal_objects_alloc_page_count - tsu.internal_objects_dealloc_page_count) AS [InternalPages],
	ROUND (CONVERT (FLOAT, (tsu.internal_objects_alloc_page_count - tsu.internal_objects_dealloc_page_count) * 8) / 1024.0, 2) AS [InternalMB],
	est.text AS [Text]
FROM
	sys.dm_db_task_space_usage tsu
	JOIN sys.dm_exec_requests er
		ON er.session_id = tsu.session_id
	CROSS APPLY sys.dm_exec_sql_text (er.sql_handle) est
/*WHERE
	-- Look for anything using 128MB or more
	((user_objects_alloc_page_count - user_objects_dealloc_page_count) +
		(internal_objects_alloc_page_count - internal_objects_dealloc_page_count)) >= 16384
		*/
ORDER BY
	((user_objects_alloc_page_count - user_objects_dealloc_page_count) +
		(internal_objects_alloc_page_count - internal_objects_dealloc_page_count)) DESC;
GO
