/*============================================================================
  File:     Transaction-Workload.sql

  Summary:  Create some transaction log usage

  Date:     March 2011

  SQL Server Versions:
		10.0.2531.00 (SS2008 SP1)
		9.00.4035.00 (SS2005 SP3)
------------------------------------------------------------------------------
  Written by Paul S. Randal, SQLskills.com

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  You may alter this code for your own *non-commercial* purposes. You may
  republish altered code as long as you give due credit.
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

-- Ever had a transaction that creates a lot more log
-- space than you thought it would?

-- Let's create one...
USE master;
GO
DROP DATABASE KeyUpdateTest;
GO

CREATE DATABASE KeyUpdateTest;
GO

USE KeyUpdateTest;
GO
ALTER DATABASE KeyUpdateTest SET RECOVERY SIMPLE;
ALTER DATABASE KeyUpdateTest SET AUTO_CREATE_STATISTICS OFF;
ALTER DATABASE KeyUpdateTest SET AUTO_UPDATE_STATISTICS OFF;
GO
CREATE TABLE test (c1 INT, c2 VARCHAR (2000));
GO
CREATE CLUSTERED INDEX test_cl ON test (c1);
GO
CHECKPOINT;
GO
INSERT INTO test VALUES (1, REPLICATE ('Paul', 500));
GO
WAITFOR DELAY '00:00:11';
GO
CHECKPOINT;
GO 

-- The checkpoints are just to clear out the transaction log

-- Check out the page
EXEC sp_allocationMetadata 'test';
GO

DBCC TRACEON (3604);
GO
DBCC PAGE (keyupdatetest, 1, XX, 2);
GO 

-- Now do an update of the 4 byte key
BEGIN TRAN
GO
UPDATE test SET c1 = 2 WHERE c1 =1;
GO 


-- How much space is being used?
-- Other window... and explain...

COMMIT TRAN;
GO

-- Look at the log records
SELECT [Current LSN], [Operation], [Context], 
 [Log Record Length], [Page ID], [Slot ID]
FROM fn_dblog (NULL, NULL);
GO 

-- Wow - not an in-place update!

-- How about this case?
DROP TABLE test;
GO

CREATE TABLE test (c1 INT, c2 VARCHAR (4000));
GO
CREATE CLUSTERED INDEX test_cl ON test (c1);
GO

INSERT INTO test VALUES (1, REPLICATE ('Paul', 1000));
GO
INSERT INTO test VALUES (2, REPLICATE ('Andy', 1000));
GO
CHECKPOINT;
GO 

-- Check out the page
EXEC sp_allocationMetadata 'test';
GO

DBCC PAGE (keyupdatetest, 1, XX, 2);
GO 

BEGIN TRAN
GO
UPDATE test SET c1 = 3 WHERE c1 =1;
GO 

COMMIT TRAN;
GO

SELECT [Current LSN], [Operation], [Context], 
 [Log Record Length], [Page ID], [Slot ID]
FROM fn_dblog (NULL, NULL);
GO

-- Also think about page splits...

-- Now, nested transactions...

BEGIN TRAN

INSERT INTO test VALUES (4, REPLICATE ('Susy', 1000));
INSERT INTO test VALUES (5, REPLICATE ('John', 1000));
GO

-- Look at usage

-- Start nested xact
BEGIN TRAN

INSERT INTO test VALUES (6, REPLICATE ('Dave', 1000));
INSERT INTO test VALUES (7, REPLICATE ('Bret', 1000));
GO

-- Look at usage

-- Commit next transaction
COMMIT TRAN;
GO

-- Look at usage
-- Any change?

-- Do final commit
COMMIT TRAN;
GO

-- Look at usage

